/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "support.h"
#include "gui.h"
#include "public.h"
#include "dirtree.h"
#include "filelist.h"
#include "queue.h"
#include "popups.h"
#include "dialog_rules.h"
#include "tag.h"
#include "custom_find.h"
#include "file_to_tag.h"
#include "configfile.h"
#include "dialog_options.h"

void
on_button_rules_up_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_up();
	queue_rules_minipreview();
}


void
on_button_rules_down_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_down();
	queue_rules_minipreview();
}


void
on_button_rules_delete_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_delete();
	queue_rules_minipreview();
}


void
on_button_rules_add_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_add();
	dialog_rules_init();
	queue_rules_minipreview();
}


void
on_button_rules_clear_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_clear();
	queue_rules_minipreview();
}


void
on_button_rules_save_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	rules_save();
}


void
on_button_rules_load_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *rules;
	gtk_clist_clear(GTK_CLIST(lookup_widget(rules, "clist_rules_rules")));
	rules_load();
	queue_rules_minipreview();
}


void
on_button_rules_back_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_close();
}


void
on_gmp3_destroy                        (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_button_gmp3_about_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *about_gmp3;

	about_gmp3=create_about_gmp3();
	gtk_widget_show(about_gmp3);
}


void
on_button_gmp3_deltag_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	extern gchar *active_tag;
	gchar *filename;
	gint row=-1;
	gchar status[4096];
	gint numrows=-1;

/* Delte either one tag (the active tag) or all tags in queue */
	if (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON(lookup_widget (gmp3, _("check_gmp3_proceed_on_queue")))))
	{
		while ((gtk_clist_get_row_data (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), ++numrows)));
		
		while ((gtk_clist_get_row_data (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), ++row)))
		{
/* Get next filename from queue */
			gtk_clist_get_text (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), row, 0, &filename);
			
/* Update status & progressbar */
			sprintf(status, "Deleting tags: %s", filename);
			gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), status);
			gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), (gfloat)100/numrows*row*0.01);
			while(g_main_iteration(FALSE));
			
/* ...and delete the tag */
			deltag(filename);
		}
	}
	else
		deltag(active_tag);
	

/* Update status & progressbar */
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
	updatetag(active_tag, 0);
}


void
on_button_gmp3_settag_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	extern gchar *active_tag;
	gchar *filename;
	gint row=-1;
	gchar status[4096];
	gint numrows=-1;
	
/* Set either one tag (the active tag) or all tags in queue */
	if (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON(lookup_widget (gmp3, _("check_gmp3_proceed_on_queue")))))
	{
		while ((gtk_clist_get_row_data (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), ++numrows)));
		
		while ((gtk_clist_get_row_data (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), ++row)))
		{
/* Get next filename from queue */
			gtk_clist_get_text (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), row, 0, &filename);

/* Update status & progressbar */
			sprintf(status, "Saving tags: %s", filename);
			gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), status);
			gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), (gfloat)100/numrows*row*0.01);
			while(g_main_iteration(FALSE));
			
/* ...and save the tag */
			settag(filename);
		}
	}
	else
		settag(active_tag);
	
/* Update status & progressbar */
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (gmp3, _("check_gmp3_proceed_on_queue"))),FALSE);
}


void
on_ctree_gmp3_directories_select_row   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	extern gchar *current_dir;
	GtkCTreeNode *node;
	
/* set new directory */
	node = gtk_ctree_node_nth (GTK_CTREE (lookup_widget (gmp3, _("ctree_gmp3_directories"))), row);
	strcpy(current_dir, gtk_ctree_node_get_row_data (GTK_CTREE (lookup_widget (gmp3, _("ctree_gmp3_directories"))), node));
	
/* set status and update filelist */
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Scanning directory, please wait...");
	while(g_main_iteration(FALSE));
	update_file_list ();
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_ctree_gmp3_directories_tree_expand  (GtkCTree        *ctree,
                                        GList           *node,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	gint i = 0;
	
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Scanning directories, please wait...");
	while(g_main_iteration(FALSE));

	while (node->prev)
	{
/* count to clicked row. */
		++i;
		node = node->prev;
	}
/* Add a new branch to the clicked node. */
	add_directory_ctree (ctree, i);

	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_clist_gmp3_iconlist_select_icon     (GnomeIconList   *gnomeiconlist,
                                        gint             arg1,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	extern gchar *current_dir;
	extern gchar *active_tag;
	sprintf(active_tag, "%s/%s", current_dir,
		(char *)gnome_icon_list_get_icon_data (GNOME_ICON_LIST (lookup_widget (gmp3, _("clist_gmp3_iconlist"))), arg1));
	updatetag(active_tag, 0);
}


gboolean
on_clist_gmp3_queue_button_press_event (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	extern GtkWidget *popup_gmp3_queue;

	if( event->button == 3 )
	{
		clist_gmp3_queue_rightclicked(event->x, event->y);
		gtk_menu_popup( GTK_MENU( popup_gmp3_queue ),
			NULL, NULL, NULL, NULL, event->button, event->time );
		return TRUE;
	}
	
 	return FALSE;
}



void
on_queue_remove_item_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_remove();
}



void
on_queue_select_done_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("done");
}



gint
on_splash_timeout                      (gpointer         user_data)
{
	extern GtkWidget *splash;
	extern GtkWidget *gmp3;
	
	gtk_widget_hide(splash);
	gtk_widget_destroy(splash);
// Show mainwindow
	gtk_widget_show (gmp3);
	return (0);
}



void
on_queue_select_errors_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("error");
}


void
on_clist_gmp3_queue_select_row         (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	extern gchar *active_tag;
	gchar *foo;
	GList* node = NULL;
	gint i=0;
	
	gtk_clist_get_text (GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), row, 0, &foo);
	strcpy(active_tag, foo);
	
	node = g_list_first(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")))->selection);
	while (node)
	{
		if(++i>1) break;
		node = g_list_next(node);
	}

	if(i>1)
		updatetag(active_tag, 1);
	else
		updatetag(active_tag, 0);
	
	filename_to_tag_preview();
	directory_to_tag_preview();
}


void
on_clist_gmp3_queue_click_column       (GtkCList        *clist,
                                        gint             column,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	gtk_clist_set_sort_column(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))), column);
	gtk_clist_sort(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));
}


void
on_button_about_gmp3_ok_clicked        (GtkObject       *object,
                                        gpointer         user_data)
{
	extern GtkWidget *about_gmp3;
	
	gtk_widget_hide(about_gmp3);
	gtk_widget_destroy(about_gmp3);
}


gboolean
on_clist_gmp3_iconlist_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	extern GtkWidget *popup_gmp3_filelist;

	if( event->button == 3 )
	{
		clist_gmp3_queue_rightclicked(event->x, event->y);
		gtk_menu_popup( GTK_MENU( popup_gmp3_filelist ),
			NULL, NULL, NULL, NULL, event->button, event->time );
		return TRUE;
	}
	
	return FALSE;
}


void
on_filelist_rename_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_filelist_rename_file();
}


void
on_filelist_refresh_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_filelist_refresh();
}


void
on_queue_rename_item_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	queue_rules_apply_selected(FALSE);
	queue_proceed_selected();
	update_file_list ();
}


void
on_queue_select_todo_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("todo");
}


gboolean
on_clist_gmp3_queue_key_press_event    (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	if(event->keyval==65535)
		popup_gmp3_queue_remove();
	return TRUE;
}


gboolean
on_clist_options_freedb_key_press_event
                                        (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	if(event->keyval==65535)
		options_freedb_server_delete();
	return TRUE;
}


void
on_button_options_freedb_add_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
	options_freedb_server_add();
}


void
on_button_options_freedb_delete_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	options_freedb_server_delete();
}


void
on_button_options_freedb_up_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_options_freedb_server_up();
}


void
on_button_options_freedb_down_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_options_freedb_server_down();
}


void
on_button_options_cancel_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *options;
	gtk_widget_destroy(options);
}


void
on_button_options_ok_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *options;
	options_save();
	gtk_widget_destroy(options);
}


void
on_button_gmp3_options_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *options;
	gtk_widget_show(options=create_options());
	options_load();
}


void
on_button_gmp3_clearqueue_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	popup_gmp3_queue_clear();
}


void
on_button_gmp3_selectdir_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	select_file_list();
}


void
on_button_gmp3_addfiles_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Adding files to queue... please wait.");
	while(g_main_iteration(FALSE));
	queue_files_add();
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_button_gmp3_addrecursive_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	extern gchar *current_dir;
	extern GtkWidget *gmp3;
	
	queue_files_add_recursive(current_dir);
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_button_gmp3_rename_editrules_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	dialog_rules_create();
	queue_rules_minipreview();
}


void
on_button_gmp3_rename_renamefiles_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	
	queue_rules_apply(FALSE);
	queue_proceed();
	
	update_file_list ();
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Done renaming files! See queue for statusinfos.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_button_gmp3_rename_preview_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	
	queue_rules_apply(TRUE);
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Preview done! (see queue)");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_button_gmp3_file2tag_settags_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;

	directory_to_tag_proceed();
	filename_to_tag_proceed();
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), "Ready.");
	gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), 0);
}


void
on_check_options_freedb_proxy_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	extern GtkWidget *options;
	gtk_widget_set_sensitive(
		GTK_WIDGET(lookup_widget(options, "entry_options_freedb_proxy")),
		!GTK_WIDGET_IS_SENSITIVE(lookup_widget(options, "entry_options_freedb_proxy")));
	gtk_widget_set_sensitive(
		GTK_WIDGET(lookup_widget(options, "entry_options_freedb_proxy_port")),
		!GTK_WIDGET_IS_SENSITIVE(lookup_widget(options, "entry_options_freedb_proxy_port")));
}


void
on_radio_options_freedb_mode_http_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	extern GtkWidget *options;
	gtk_widget_set_sensitive(
		GTK_WIDGET(lookup_widget(options, "entry_options_freedb_path")),
		!GTK_WIDGET_IS_SENSITIVE(lookup_widget(options, "entry_options_freedb_path")));
}

void
on_queue_preview_item_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	queue_rules_apply_selected(TRUE);
}


void
on_ctree_gmp3_directories_tree_collapse
                                        (GtkCTree        *ctree,
                                        GList           *node,
                                        gpointer         user_data)
{
	gint i=0;
	
	while (node->prev)
	{
// get active row.
		++i;
		node = node->prev;
	}
	
	if(GTK_CTREE_NODE(GTK_CTREE_ROW(gtk_ctree_node_nth(ctree, i))->children))
		collapse_directory_ctree (ctree, i);
}


void
on_queue_invert_selection_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("invert");
}


void
on_queue_select_anytag_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("id3v1");
	popup_gmp3_queue_select("id3v2");
}


void
on_queue_select_notag_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("notag");
}


void
on_queue_select_id3v1_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("id3v1");
}


void
on_queue_select_id3v2_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_queue_select("id3v2");
}


void
on_entry_gmp3_file2tag_file_changed    (GtkEditable     *editable,
                                        gpointer         user_data)
{
	filename_to_tag_preview();
	directory_to_tag_preview();
}


void
on_entry_gmp3_file2tag_directory_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
	filename_to_tag_preview();
	directory_to_tag_preview();
}


void
on_button_gmp3_rename_file_cancel_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *popup_rename_file;
	gtk_widget_destroy(popup_rename_file);
}


void
on_button_gmp3_rename_file_ok_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	popup_gmp3_filelist_rename_file_ok();
}


void
on_entry_gmp3_rename_file_name_activate
                                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
	popup_gmp3_filelist_rename_file_ok();
}


void
on_button_gmp3_rename_directory_cancel_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *popup_rename_directory;
	gtk_widget_destroy(popup_rename_directory);
}


void
on_button_gmp3_rename_directory_ok_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	popup_gmp3_dirtree_rename_directory_ok();
}


void
on_entry_gmp3_rename_directory_name_activate
                                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
	popup_gmp3_dirtree_rename_directory_ok();
}


gboolean
on_ctree_gmp3_directories_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	extern GtkWidget *gmp3;
	extern gint lastclick_x;
	extern gint lastclick_y;
	extern GtkWidget *popup_gmp3_dirtree;
	gint row, col;

	if( event->button == 3 )
	{
		lastclick_x = event->x;
		lastclick_y = event->y;
		
		gtk_clist_get_selection_info(GTK_CLIST (lookup_widget (gmp3, "ctree_gmp3_directories")), event->x, event->y, &row, &col);
		gtk_clist_select_row(GTK_CLIST (lookup_widget (gmp3, "ctree_gmp3_directories")), row, col);
		gtk_menu_popup( GTK_MENU( popup_gmp3_dirtree ),
			NULL, NULL, NULL, NULL, event->button, event->time );
		return TRUE;
	}
	
	return FALSE;
}


void
on_popup_gmp3_dirtree_rename_directory_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popup_gmp3_dirtree_rename_directory();
}

void
on_button_gmp3_findandlists_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	extern GtkWidget *custom_find;
	custom_find = create_custom_find();
	gtk_widget_show(custom_find);
}


void
on_button_gmp3_help_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_button_custom_find_cancel_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_button_custom_find_find_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	extern gchar *current_dir;
	GList *foundfiles;
	FoundRow *foundrow;
	gint numrows=0;
	
	foundfiles = g_list_last(custom_find_find(current_dir));
	numrows = g_list_length(foundfiles);
	
	while(numrows-->=0)
	{
		foundrow = (FoundRow *)foundfiles->data;
		printf("%i: %s - %s\n", numrows, foundrow->directory, foundrow->filename);
		foundfiles = g_list_previous(foundfiles);
	}
	
}
