/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "custom_find.h"
#include "tag.h"
#include "id3v1.h"
#include "id3v2.3.h"


/*
* Return 1 if filepathname is a directory, otherwise 0
*/
static int
is_a_dir (gchar *filepathname)
{
	struct stat pstat;
	stat (filepathname, &pstat);
	return (S_ISDIR (pstat.st_mode));
}





GList *custom_find_find(gchar *dir)
{
	extern GtkWidget *custom_find;
	
	GList *foundfiles = g_list_alloc();
	FoundRow *foundrow = NULL;
	
	struct dirent *pdirent = NULL;
	DIR *pdir = NULL;
	gchar current[2048];

/* Open directory */
	if((pdir = opendir (dir)) == NULL) 
	{
		printf("200 - Error opening requested path ('%s')!\n", dir);
		free(pdir);
		exit(1);
	}
	
/* get entry from directory */
	while((pdirent = readdir (pdir)) != NULL)
	{
		sprintf (current, "%s%s", dir, pdirent->d_name);
		if ( !is_a_dir (current) )
		{
			foundrow = malloc(sizeof(FoundRow));
			strncpy(foundrow->directory, dir, 2047); 
			strncpy(foundrow->filename, pdirent->d_name, 1023);
			foundfiles = g_list_append(foundfiles, (gpointer)foundrow);
		}
		else {
//			custom_find_find(bar);
		}
	}
	closedir(pdir);
	
	return foundfiles;
}








gboolean
has_v1tag(gchar *filename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	if(get_id3v1_tag(tag, filename)==0)
	{
		free(tag);
		return TRUE;
	}
	free(tag);
	
	return FALSE;
}





gboolean
has_v2tag(gchar *filename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	if(get_id3v2_tag(tag, filename)==0)
	{
		free(tag);
		return TRUE;
	}
	free(tag);
	
	return FALSE;
}






gboolean
has_tag(gchar *filename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	if( (get_id3v1_tag(tag, filename)==0)
		&& (get_id3v2_tag(tag, filename)==0) )
	{
		free(tag);
		return TRUE;
	}
	free(tag);
	
	return FALSE;
}





gboolean
has_empty_v1tag(gchar *filename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	
	memset(tag->artist, '\0', 1024);
	memset(tag->title, '\0', 1024);
	memset(tag->album, '\0', 1024);
	memset(tag->year, '\0', 1024);
	memset(tag->comment, '\0', 1024);
	
	if( (get_id3v1_tag(tag, filename)==0)
		&& (*tag->artist!='\0'
		|| *tag->title!='\0'
		|| *tag->album!='\0'
		|| *tag->year!='\0'
		|| *tag->comment!='\0') )
	{
		free(tag);
		return TRUE;
	}
	free(tag);
	
	return FALSE;
}




gboolean
has_empty_v2tag(gchar *filename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	
	memset(tag->artist, '\0', 1024);
	memset(tag->title, '\0', 1024);
	memset(tag->album, '\0', 1024);
	memset(tag->year, '\0', 1024);
	memset(tag->comment, '\0', 1024);
	
	if( (get_id3v2_tag(tag, filename)==0)
		&& (*tag->artist!='\0'
		|| *tag->title!='\0'
		|| *tag->album!='\0'
		|| *tag->year!='\0'
		|| *tag->comment!='\0') )
	{
		free(tag);
		return TRUE;
	}
	free(tag);
	
	return FALSE;
}




/*
 * Similarity check.
 * (c) 2002 by Samuel Abels.
 *
 * Determines if string1 is similar to string2
 * string1, string2 are the strings to be compared
 * tolerance is the tolerance (1 to 100)
 * returns: TRUE if similar, FALSE if not.
 */
gboolean
is_similar(gchar *string1, gchar *string2, gint tolerance)
{
	gchar *pstring1 = string1;
	gchar *pstring2 = string2;
	gint matches = 0;
	gchar current[2];
	
// the smaller string has to come first
	if(strlen(string1) > strlen(string2))
	{
		pstring1=string1;
		string1=string2;
		string2=pstring1;
	}

// length too different? (the "+2" is to make this less sensible, try to experiment with)
	if( (strlen(string2) - strlen(string1))
		> ((gfloat)strlen(string1)/100*tolerance)+2 )
		return FALSE;

// now to the "real" similarity check.
// i will get the first two chars of string 1, and look if this combination is found anywhere in string2, too.
// if yes -> increase matches by one.
// Then go get two chars from string 1 again, beginning with the second character of string 1, seek this in string2, and so on...
// When we're done, matches should be max. len of string1.
	pstring1 = string1;
	while(*pstring1!='\0')
	{
		current[0]=*pstring1;
		current[1]=*(pstring1+1);
		
		pstring2 = string2;
		while(*pstring2!='\0')
		{
			if( (current[0]==*pstring2)
				&& (current[1]==*(pstring2+1)) )
			{
				++matches;
				break;
			}
			++pstring2;
		}
		++pstring1;
	}

// if: matches > the size of (string1 - some tolerance) return TRUE
	if( matches >= (gfloat)strlen(string1) - (gfloat)strlen(string1)/100*tolerance )
		return TRUE;
	
	return FALSE;
}