/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "config.h"
#include "callbacks.h"
#include "support.h"
#include "dirtree.h"
#include "public.h"

#if !S_IXUGO
#define S_IXUGO        (S_IXUSR | S_IXGRP | S_IXOTH)
#endif

extern Stack ctree_directories_data;

/*
* Return 1 is filepathname is a directory, otherwise 0
*/
static int
is_a_dir (char *filepathname)
{
	struct stat pstat;
	stat (filepathname, &pstat);
	return (S_ISDIR (pstat.st_mode));
}



/*
* Print out the initial content of the directory ctree box
*/
void
init_directory_ctree (GtkCTree *ctree)
{
	extern GtkWidget *gmp3;
	GtkCTreeNode *parentnode;
	GtkCTreeNode *dummynode=NULL;
	gchar *column[1];
	gchar *temp=NULL;


	GdkColormap *colormap=NULL;
	GdkPixmap *gdkpixmap_close=NULL;
	GdkBitmap *mask_close=NULL;
	GdkPixmap *gdkpixmap_open=NULL;
	GdkBitmap *mask_open=NULL;

	gtk_ctree_set_indent(ctree, 10);
	setenv("PWD", "/", 1);
	
	colormap = gtk_widget_get_colormap (gmp3);
	
// Init parent node
	column[0] = mem_allocate (&ctree_directories_data);
	sprintf(column[0], "/");

	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/folder_opened.xpm", NULL);
	gdkpixmap_open = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_open, NULL, temp);
	free(temp);
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/folder_closed.xpm", NULL);
	gdkpixmap_close = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_close, NULL, temp);
	free(temp);

	parentnode = gtk_ctree_insert_node(ctree, NULL, NULL, column, 5, gdkpixmap_close, mask_close, gdkpixmap_open, mask_open, FALSE, FALSE);
	gtk_ctree_node_set_row_data(ctree, parentnode, (gpointer) "/");

// and insert one dummy node
	column[0]=mem_allocate (&ctree_directories_data);
	sprintf(column[0], "*");
	dummynode = gtk_ctree_insert_node(GTK_CTREE (ctree), parentnode, NULL, column, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_row_data(ctree, dummynode, (gpointer) "*");

	gtk_ctree_expand(ctree,parentnode);
}



/*
* Add a branch to the directory tree
*/
void
add_directory_ctree (GtkCTree *ctree, gint row)
{
	extern GtkWidget *gmp3;
	GtkCTreeNode *node=NULL;
	GtkCTreeNode *subnode=NULL;
	GtkCTreeNode *dummynode=NULL;
	gchar *column[1];

	struct dirent *pdirent=NULL;
	DIR *pdir=NULL, *pdir2=NULL;
	struct stat filestatus;

	gchar *dir=NULL;
	gchar *temp=NULL;
	gchar *rowdata=NULL;
	gchar filepathname[2000];

	GdkColormap *colormap=NULL;
	GdkPixmap *gdkpixmap_close=NULL;
	GdkBitmap *mask_close=NULL;
	GdkPixmap *gdkpixmap_open=NULL;
	GdkBitmap *mask_open=NULL;
	GdkPixmap *gdkpixmap_locked=NULL;
	GdkBitmap *mask_locked=NULL;

	colormap = gtk_widget_get_colormap (gmp3);

	
// if it exists, delete dummy entry. this will collapse the branch!!
	node=gtk_ctree_node_nth(ctree, row+1);
	dir=(char *)gtk_ctree_node_get_row_data(ctree, node);
	if(!strcmp(dir, (char *)"*"))
		gtk_ctree_remove_node(ctree, node);
	else
		return;

	gtk_clist_freeze(GTK_CLIST(ctree));
	
 	node=gtk_ctree_node_nth(ctree, row);
	dir=(char *)gtk_ctree_node_get_row_data(ctree, node);
// Open directory
	if ((pdir = opendir (dir)) == NULL) 
	{
		printf("Error opening requested path ('%s')!\n", dir);
		exit(1);
	}

	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/folder_opened.xpm", NULL);
	gdkpixmap_open = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_open, NULL, temp);
	free(temp);
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/folder_closed.xpm", NULL);
	gdkpixmap_close = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_close, NULL, temp);
	free(temp);
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/folder_locked.xpm", NULL);
	gdkpixmap_locked = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_locked, NULL, temp);
	free(temp);
// Add each direct subdir to the tree.
	while ((pdirent = readdir (pdir)) != NULL)
	{
		rowdata = mem_allocate (&ctree_directories_data);
		sprintf (rowdata, "%s%s", dir, pdirent->d_name);
		if ( is_a_dir (rowdata)
		    && strcmp (pdirent->d_name, "..")
		    && strcmp (pdirent->d_name, ".") )
		{
			column[0] = mem_allocate (&ctree_directories_data);
			strncpy(column[0], pdirent->d_name, strlen(pdirent->d_name));
			
			if ((pdir2 = opendir (rowdata)) != NULL)
				subnode = gtk_ctree_insert_node(GTK_CTREE (ctree), node, NULL, column, 5, gdkpixmap_close, mask_close, gdkpixmap_open, mask_open, FALSE, FALSE);
			else
				subnode = gtk_ctree_insert_node(GTK_CTREE (ctree), node, NULL, column, 5, gdkpixmap_locked, mask_locked, gdkpixmap_locked, mask_locked, FALSE, FALSE);
			closedir(pdir2);
			
			strncpy(rowdata+strlen(rowdata), "/\0", 2);
			gtk_ctree_node_set_row_data(ctree, subnode, (gpointer) rowdata);
// now, we check if we have rights on the added node.
			stat (rowdata, &filestatus);
			if ((pdir2 = opendir (rowdata)) != NULL)
			{
				dummynode=NULL;
// so we have the rights, look for subdirs. if any exist, enable the "+" box by adding a dummy-subnode.
				while ((pdirent = readdir (pdir2)) != NULL)
				{
					sprintf (filepathname, "%s%s", rowdata, pdirent->d_name);
					if ( is_a_dir (filepathname)
						&& strncmp (pdirent->d_name, "..", 2)
						&& strncmp (pdirent->d_name, ".", 1)
						&& dummynode==NULL )
					{
						dummynode = gtk_ctree_insert_node(GTK_CTREE (ctree), subnode, NULL, column, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
						gtk_ctree_node_set_row_data(ctree, dummynode, (gpointer) "*");
						break;
					}
				}
			}
			closedir(pdir2);
		}
	}
	closedir(pdir);
	gtk_ctree_sort_recursive(ctree, node);
	gtk_ctree_expand(ctree,node);
	gtk_clist_thaw(GTK_CLIST(ctree));
}





/*
* Remove a branch of the directory tree
*/
void
collapse_directory_ctree (GtkCTree *ctree, gint row)
{
	GtkCTreeNode *node=NULL, *dummynode=NULL;
	GtkCTreeNode *child=NULL;
	gchar *column[1];

	gtk_clist_freeze(GTK_CLIST(ctree));

	node = gtk_ctree_node_nth(ctree, row);
	child = GTK_CTREE_ROW(node)->children;
	
// remove all children
	while(child)
	{
		gtk_ctree_remove_node(ctree, child);
		child = GTK_CTREE_ROW(node)->children;
	}
// and add a dummy node again.
	column[0]=calloc(2,2);
	*column[0]='*';
	dummynode = gtk_ctree_insert_node(GTK_CTREE (ctree), node, NULL, column, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_row_data(ctree, dummynode, (gpointer) "*");
	
	free(column[0]);
	gtk_clist_thaw(GTK_CLIST(ctree));
}