/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "freedb.h"

int freedb_get(CDInfo *cdinfo)
{
	struct sockaddr_in socket_address;
	int socket_number;

	char bin[4096];		// Binary in data
	char bout[4096];	// Binary out data
	char buf[4096];

	int foo, i;

// Get a Socket number
	if((socket_number = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("Fehler: Konnte keinen INET Socket ffnen!");
		return -1;
	}

	socket_address.sin_family = AF_INET;
	//socket_address.sin_addr.s_addr = dir_INET("freedb.freedb.org");	// address of the FreeDB host
	socket_address.sin_port = htons(888);				// port of the FreeDB protocol

// Connect to the server...
	if(connect(socket_number, (struct sockaddr *)&socket_address, sizeof(socket_address)) == -1)
		perror("Can't connect to FreeDB host!");

// Read hello string. Read returns the number of received bytes or -1 if unsuccesful.
	foo = read(socket_number, bin, sizeof(bin));
	if(foo < 0)
	{
		perror("Can't read from FreeDB host!");
		close(socket_number);
		return -1;
	}

// set the byte after the last received byte to zero.
	bin[foo] = 0;
// And print out what the server said.
	printf("<< %s", bin);
	if(strncmp(bin, "200", 3) && strncmp(bin, "201", 3))
	{
		perror("Unknown server reply!");
		close(socket_number);
		return -1;
	}

// Send my hello string
	sprintf(bout, "cddb hello %s %s gmp3 0.1 \n", getenv("USER"), getenv("HOSTNAME"));
	foo = write(socket_number, bout, strlen(bout));
	if(foo<0) {
		perror("Error while sending hello string to the server!");
		close(socket_number);
		return -1;
	}
// And print out what i said.
	printf(">> %s",bout);

// Read the answer of my salute
	foo = read(socket_number, bin, sizeof(bin));
	if(foo<0) {
		perror("Error while hello reply!");
		close(socket_number);
		return -1;
	}
// NULL-terminate the answer, again.
	bin[foo] = 0;
// And print it out to the screen...
	printf("<< %s",bin);
	if(strncmp(bin, "200", 3))
	{
		perror("Unknown hello reply!");
		close(socket_number);
		return -1;
	}

// Now I'm preparing the CDInfo for sending
	sprintf(bout, "cddb query %08lx %08lx ", cdinfo->id, cdinfo->tracks);
	for(i=0; i<cdinfo->tracks; i++)
		sprintf((char *)&bout[strlen(bout)], "%lu ", cdinfo->track_offset[i]);
	sprintf((char *)&bout[strlen(bout)], "%u\n", cdinfo->seconds);

// And send it to the server!
	foo = write(socket_number, bout, strlen(bout));
	printf(">> %s", bout);
	if(foo<0) {
		perror("Error while sending FreeDB request!");
		close(socket_number);
		return -1;
	}

// Ok! i now read the answer
	bzero(bin, sizeof(bin));
	foo = read(socket_number, bin, sizeof(bin));
	if(foo<0) {
		perror("Error while reading CD information");
		close(socket_number);
		return -1;
	}
// NULL terminate
	bin[foo] = 0;
	printf("<< %s", bin);

// I got a reply in format "200 <category> <discid> <title>" if match is found, or
// "211 close matches found" followed by a cd list in format "categ discid title\n"
	if(strncmp(bin, "200", 3) && strncmp(bin, "211", 3)) {
		printf("Unknown disc!\n");
		close(socket_number);
		return -1;
	}

/*
* -------- First we'll handle "best case" --------
 */
	if(strncmp(bin, "200", 3))
	{
		printf("Exact match!");
// I get the genre of the CD into buf
		sscanf(bin, "%s%s", bout, buf);
		realloc(cdinfo->genre, strlen(buf)+1);
		strcpy(cdinfo->genre, buf);

// cut away anything before the third space in bin, so what we got is the title
		strcpy(bin, strchr(strchr(strchr(bin, ' ')+1, ' ')+1, ' ')+1);
// Removes spaces and some other trash from the end of the string
		while(bin[strlen(bin)-1]<=32 && strlen(bin)>0)
			bin[strlen(bin)-1] = 0;
		realloc(cdinfo->cdtitle, strlen(bin)+1);
		strcpy(cdinfo ->cdtitle, bin);
	}
	else
	{
// so if we DON'T have our "best case", we still have close matches.
// ... which are not yet implemented :-).
	}

// Request the name of all the tracks
	sprintf(bout, "CDDB READ %s %08lx\n", cdinfo->genre, cdinfo->id);
	foo = write(socket_number, bout, strlen(bout));
	printf(">> %s", bout);
	if(foo<0) {
		perror("Error while sending cd info request!");
		close(socket_number);
		return -1;
	}
// We get the answer of the query
	bzero(bin, sizeof(bin));
// We need to get char by char, to the first occurence of a line feed, so we get only the first line
// the first line contains NOT a title (and there may be more lines like that), but some header informations
// we don't care about.
	for(i=0; i<sizeof(bin); i++) {
		if( (strlen(buf)>0)
			&& (!strncmp((char *)&buf[strlen(buf)-2], "\r\n", 2)) )
			break;
		foo = read(socket_number, (char *)&buf[i], 1);
	}
	if(foo<0) {
		perror("Error while receiving cd data!");
		close(socket_number);
		return -1;
	}
	bin[foo] = 0;
	printf("<< %s", bin);
	if(strncmp(bin, "210", 3)) {
		printf("Unknown answer!\n");
		close(socket_number);
		return -1;
	}

// so the server sent a valid reply, we can get all titles now.
// we only care about lines beginning with "TTITLE", that will skip all header lines and stuff.
// a line containing nothing but a dot (plus a linefeed) means end of data.
	i = 0;
	while(strcmp(bin, ".\r\n"))
	{
		bzero(bin,sizeof(bin));
// Get one line
		for(i=0; i<sizeof(bin); i++) {
			if( (strlen(buf)>0)
				&& (!strncmp((char *)&buf[strlen(buf)-2], "\r\n", 2)) )
				break;
		foo = read(socket_number, (char *)&buf[i], 1);
		}
// If an error occured, quit
		if(foo<0) {
			perror("Error while receiving track information!");
			close(socket_number);
			return -1;
		}
// Null-terminate line
		bin[foo] = 0;
		printf("<< %s", bin);

		if(!strncmp(bin, "TTITLE", 6))
		{
// if the line contains a tracktitle, remove the linefeed.
			while((bin[strlen(bin)-1]=='\n' || bin[strlen(bin)-1]=='\r')
				&& strlen(bin)>0)
				bin[strlen(bin)-1] = 0;
// the line has the format: "TTITLE=<title>"
// so we save anything after the "=" as tracktitle.
			cdinfo->track[i] = (char *)malloc(strlen(bin)-5);
			strcpy(cdinfo->track[i], strchr(bin, '=')+1);
			i++;
		}

		if(!strncmp(bin, "DYEAR", 5))
		{
// if the line contains the year, remove the linefeed.
			while((bin[strlen(bin)-1]=='\n' || bin[strlen(bin)-1]=='\r')
				&& strlen(bin)>0)
				bin[strlen(bin)-1] = 0;
// the line has the format: "DYEAR=<title>"
// so we save anything after the "=" as year.
			cdinfo->year = (char *)malloc(5);
			strncpy(cdinfo->year, strchr(bin, '=')+1, 4);
		}
	}

// We're finished, send the "quit" command to the server.
	sprintf(bout, "QUIT\n");
foo=write(socket_number, bout, strlen(bout));
	printf(">> %s", bout);
	if(foo<0) {
		perror("Error while sending the quit command. Should not be important to you :-).");
		close(socket_number);
		return -2;
	}
// We get the servers answer (=quit confirm) to be conform with the freedb specifications.
// In fact, we don't really care what he says now :-).
	bzero(bin,sizeof(bin));
	foo=read(socket_number,bin,sizeof(bin));
	if(foo<0) {
		perror("Error while receiving the quit confirmation from the server.");
		close(socket_number);
		return -2;
	}
	bin[foo] = 0;
	printf("<< %s", bin);

// Close the socket connection with my host
	close(socket_number);
	return 0;
}

