/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "support.h"		/* contains some functions glade needs to create the gui */
#include "gui.h"			/* contains functions to create the gui */
#include "public.h"			/* contains stack functions */
#include "dirtree.h"		/* update and change the ctree */
#include "filelist.h"		/* update and change clist1, wich is not the query list */
#include "queue.h"			/* update and change clist1, wich is not the query list */
#include "callbacks.h"		/* any callback funcs */
#include "configfile.h"		/* to get the splashscreen option */

/* Init global variables for the windows and dialogs */
GtkWidget *splash;
GtkWidget *gmp3;
GtkWidget *options;
GtkWidget *rules;
GtkWidget *popup_gmp3_queue;
GtkWidget *popup_gmp3_filelist;
GtkWidget *popup_gmp3_dirtree;
GtkWidget *popup_rename_file;
GtkWidget *popup_rename_directory;
GtkWidget *custom_find;
GtkWidget *custom_show;
GtkWidget *about_gmp3;

/* These are stacks for an easier and more secure memoryhandling (defined in "public.h") */
Stack clist_files_data;
Stack clist_queue_data;
Stack clist_rules_data;
Stack clist_options_data;
Stack ctree_directories_data;
Stack free_on_exit;

/* Other globals */
gchar *current_dir;					/* Always will contain the directory name which is active for the filelist */
gchar *active_tag;					/* Will contain the full filename of the tag currently shown in the tageditor */
gint lastclick_x, lastclick_y;		/* Last click, is needed to determine the clickposition for popupmenus */


int
main (int argc, char *argv[])
{
	gchar *nosplash=NULL;
	
#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	current_dir = calloc(2000, sizeof(gchar));
	active_tag = calloc(2000, sizeof(gchar));
// Initialize stacks
	mem_init(&clist_files_data);
	mem_init(&clist_queue_data);
	mem_init(&clist_rules_data);
	mem_init(&clist_options_data);
	mem_init(&ctree_directories_data);
	mem_init(&free_on_exit);

	gnome_init ("gmp3", VERSION, argc, argv);

// Create widgets
	popup_gmp3_queue = create_popup_gmp3_queue();
	popup_gmp3_filelist = create_popup_gmp3_filelist();
	popup_gmp3_dirtree = create_popup_gmp3_dirtree();
	gmp3 = create_gmp3 ();
	splash = create_splash();
	
// Show splashscreen, if enabled
	nosplash = calloc(2000, sizeof(gchar));
	option_get("NO_SPLASH", nosplash);
	if(strncmp(nosplash, "FALSE", 5)==0)
	{
		gtk_widget_show (splash);
		gtk_timeout_add(3000, on_splash_timeout, NULL);
	}
	else
		gtk_widget_show (gmp3);
	free(nosplash);
	
// Init the directorytree and the queue list.	
	gtk_clist_columns_autosize(GTK_CLIST (lookup_widget (gmp3, "clist_gmp3_queue")));
	init_directory_ctree(GTK_CTREE (lookup_widget (gmp3, "ctree_gmp3_directories")));

	gtk_main ();
	
// deallocate memory and quit
	mem_free(&clist_files_data);
	mem_free(&clist_queue_data);
	mem_free(&clist_rules_data);
	mem_free(&clist_options_data);
	mem_free(&ctree_directories_data);
	mem_free(&free_on_exit);

	free(current_dir);
	free(active_tag);
	return 0;
}
