/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "support.h"
#include "queue.h"
#include "public.h"
#include "rules.h"
#include "config.h"
#include "id3v1.h"
#include "id3v2.3.h"



/*
* Return 1 is filepathname is a directory, otherwise 0
*/
static int
is_a_dir (gchar *filepathname)
{
	struct stat pstat;
	stat (filepathname, &pstat);
	return (S_ISDIR (pstat.st_mode));
}



void queue_proceed(void)
{
	extern GtkWidget *gmp3;
	GtkCList *clist_queue;
	gint queuerow = -1;
	gchar *fileold, *filenew, *directory, *foo;
	GdkColormap *colormap=NULL;
	GdkPixmap *gdkpixmap=NULL;
	GdkBitmap *mask=NULL;
	gchar *rowdata=NULL;
	gchar *temp=NULL;
	gchar status[4096];
	gint numrows=-1;

	colormap = gtk_widget_get_colormap (gmp3);
	clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));
	
	directory = calloc(2000,sizeof(gchar));
	fileold = calloc(2000,sizeof(gchar));
	filenew = calloc(2000,sizeof(gchar));

	while (gtk_clist_get_row_data (clist_queue, ++numrows));

	while ((gtk_clist_get_text (clist_queue, ++queuerow, 0, &foo)))
	{
		if(strcmp(foo, "done")!=0)
		{
			gtk_clist_get_text (clist_queue, queuerow, 0, &foo);
	
			sprintf(status, "Renaming files: %s", foo);
			gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), status);
			gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), (gfloat)100/numrows*queuerow*0.01);
			while(g_main_iteration(FALSE));
		
			sprintf(directory, "%s", foo);
			*strrchr(directory,'/')='\0';
			sprintf(fileold, "%s", foo);
			
			gtk_clist_get_text (clist_queue, queuerow, 2, &foo);
			sprintf(filenew, "%s/%s", directory, foo);

			rowdata = gtk_clist_get_row_data(clist_queue, queuerow);

			if(access(fileold, W_OK))
			{
				temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/locked.xpm", NULL);
				gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
				free(temp);
				gtk_clist_set_pixmap(clist_queue, queuerow, 1, gdkpixmap, mask);
				sprintf(rowdata, "locked");
			}
			else
			{
				if(rename(fileold, filenew)==0)
				{
					temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/done.xpm", NULL);
					gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
					free(temp);
					gtk_clist_set_pixmap(clist_queue, queuerow, 1, gdkpixmap, mask);
					sprintf(rowdata, "done");
				}
				else
				{
					temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/error.xpm", NULL);
					gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
					free(temp);
					gtk_clist_set_pixmap(GTK_CLIST (clist_queue), queuerow, 1, gdkpixmap, mask);
					sprintf(rowdata, "error");
				}
			}
		}
	}	
	free(fileold);
	free(filenew);
	free(directory);
	gtk_clist_columns_autosize(clist_queue);
}







void queue_files_add(void)
{
	extern GtkWidget *gmp3;
	extern Stack clist_queue_data;
	extern gchar *current_dir;
	GnomeIconList *gil_files=NULL;
	GtkCList *clist_queue=NULL;
	
	gchar *filename=NULL;
	gchar *output[3];
	gchar *rowdata=NULL;
	gchar *foo=NULL, *temp=NULL;
	gchar *bar = calloc(2000, sizeof(gchar));
	gint row=0, i=-1;
	GList *selection=NULL;
	gboolean found;

	GdkColormap *colormap=NULL;
	GdkPixmap *gdkpixmap=NULL;
	GdkBitmap *mask=NULL;

	colormap = gtk_widget_get_colormap (gmp3);

	gil_files = GNOME_ICON_LIST (lookup_widget (gmp3, _("clist_gmp3_iconlist")));
	clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));

	selection = gil_files->selection;

	gtk_clist_freeze(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));
	while (selection)
	{
		row = (gint)selection->data;

		filename = gnome_icon_list_get_icon_data (gil_files, row);

// Make sure the entry is not in list, yet
		i=-1;
		found=FALSE;
		sprintf (bar, "%s%s", current_dir, filename);
		while ((gtk_clist_get_text (clist_queue, ++i, 0, &foo)))
		{
			if(strcmp(foo, bar)==0)
			{
				found = TRUE;
				break;
			}
		}
		
		if(!found) {
			output[0] = mem_allocate(&clist_queue_data);
			output[2] = mem_allocate(&clist_queue_data);
			rowdata = mem_allocate(&clist_queue_data);
			sprintf (output[0], "%s%s", current_dir, filename);
			sprintf (output[2], "Click \"Preview\" to update.");
			sprintf (rowdata, "todo");
			row = gtk_clist_append (GTK_CLIST (clist_queue), output);
			
			temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/todo.xpm", NULL);
			gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
			free(temp);
			gtk_clist_set_pixmap(GTK_CLIST (clist_queue), row, 1, gdkpixmap, mask);
			gtk_clist_set_row_data(clist_queue, row, rowdata);
		}

		selection = selection->next;
	}
	free(bar);
	gtk_clist_sort(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));
	gtk_clist_columns_autosize(clist_queue);
	gtk_clist_thaw(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));
}



/*
 ***************************************************************************
 * will apply all rules specified by the user.
 ***************************************************************************
 */
void queue_rules_apply(gboolean preview)
{
	extern GtkWidget *gmp3;
	GtkCList *clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));
	GtkCList *clist_rules = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_rules")));
	gchar *filename, *extension, *output;
	gchar *rulestring;
	gchar *option[3];
	gint queuerow = -1;
	gint rulerow;
	gchar *fullfilename;
	gchar status[4096];
	gint numrows=-1;

	gtk_clist_freeze(clist_queue);
	
	extension = calloc(2000,sizeof(gchar));
	filename = calloc(2000,sizeof(gchar));
	output = calloc(2000,sizeof(gchar));
	option[0] = calloc(2000,sizeof(gchar));
	option[1] = calloc(2000,sizeof(gchar));
	option[2] = calloc(2000,sizeof(gchar));

	while (gtk_clist_get_row_data (clist_queue, ++numrows));

	while ((gtk_clist_get_text (clist_queue, ++queuerow, 0, &fullfilename)))
	{
		sprintf(status, "Applying rules: %s", fullfilename);
		gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), status);
		gnome_appbar_set_progress(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), (gfloat)100/numrows*queuerow*0.01);
		while(g_main_iteration(FALSE));
		
		sprintf(extension, "%s", strrchr(fullfilename,'.'));
		sprintf(filename, "%s", strrchr(fullfilename,'/') + 1);
		*strrchr(filename,'.')='\0';
		
		rulerow=0;
		rulestring = gtk_clist_get_row_data (clist_rules, rulerow);
		while (rulestring)
		{
			sprintf(option[0], "%s", rulestring);
			rulestring=rulestring+strlen(rulestring)+1;
			sprintf(option[1], "%s", rulestring);
			rulestring=rulestring+strlen(rulestring)+1;
			sprintf(option[2], "%s", rulestring);

			if(!preview)
			{
// This rules can't be previewed (tag changers!)
				if(strcmp(option[0],(char *)"v1tov2")==0)
					apply_rule_v1tov2(fullfilename);
				if(strcmp(option[0],(char *)"v2tov1")==0)
					apply_rule_v2tov1(fullfilename);
				if(strcmp(option[0],(char *)"v1delete")==0)
				{
					if(access(fullfilename, W_OK)==0)
						del_id3v1_tag(fullfilename);
				}
				if(strcmp(option[0],(char *)"v2delete")==0)
				{
					if(access(fullfilename, W_OK)==0)
						del_id3v2_tag(fullfilename);
				}
				if(strcmp(option[0],(char *)"tagcleanup")==0)
					apply_rule_tagcleanup (fullfilename);
			}
			else {
// Apply this rules to all filenames in queue.
				if(strcmp(option[0],(char *)"gettag")==0)
					apply_rule_gettag (filename, fullfilename, option[1]);
				if(strcmp(option[0],(char *)"replace")==0)
					apply_rule_replace (filename, option[1], option[2]);
				if(strcmp(option[0],(char *)"format")==0)
					apply_rule_format_to_format (filename, option[1], option[2]);
				if(strcmp(option[0],(char *)"parentheses")==0)
				{
					if(strcmp(option[1],(gchar *)"all")==0)
						apply_rule_remove_parentheses (filename, TRUE);
					else
						apply_rule_remove_parentheses (filename, FALSE);
				}
				if(strcmp(option[0],(char *)"forcecc")==0)
				{
					if(strcmp(option[1],(gchar *)"only first letter to upper")==0)
						apply_rule_forcecc (filename, 1);
					else if(strcmp(option[1],(gchar *)"all words begin uppercase")==0)
						apply_rule_forcecc (filename, 2);
					else if(strcmp(option[1],(gchar *)"every character lowercase")==0)
						apply_rule_forcecc (filename, 3);
					else
						apply_rule_forcecc (filename, 4);
				}
				if(strcmp(option[0],(char *)"deletechar")==0)
					apply_rule_delete_char (filename, atoi(option[1]), atoi(option[2]));
				if(strcmp(option[0],(char *)"increase")==0)
					apply_rule_number_increase (filename, atoi(option[1]));
				if(strcmp(option[0],(char *)"stripchar")==0)
					apply_rule_strip_to_digit (filename);
				if(strcmp(option[0],(char *)"twodigit")==0)
					apply_rule_digit_to_twodigit (filename);
				if(strcmp(option[0],(char *)"insertspace")==0)
					apply_rule_insert_char_before_uppercase (filename, (char *)" ");
				if(strcmp(option[0],(char *)"stripspace")==0)
					apply_rule_strip_multiple_spaces(filename);
				if(strcmp(option[0],(char *)"killspace")==0)
				{
					if(strcmp(option[1],(gchar *)"start and end")==0)
						apply_rule_wipe_spaces (filename, 2);
					else if(strcmp(option[1],(gchar *)"start")==0)
						apply_rule_wipe_spaces (filename, 0);
					else
						apply_rule_wipe_spaces (filename, 1);
				}
			}
				
			rulestring = (char *)gtk_clist_get_row_data (clist_rules, ++rulerow);
		}
		sprintf (output, "%s%s", filename, extension);
		gtk_clist_set_text (GTK_CLIST (clist_queue), queuerow, 2, (char *)output);
	}
	free(option[2]);
	free(option[1]);
	free(option[0]);
	free(output);
	free(filename);
	free(extension);

	gtk_clist_thaw(clist_queue);
	gtk_clist_columns_autosize(clist_queue);
}







void queue_proceed_selected(void)
{
	extern GtkWidget *gmp3;
	GtkCList *clist_queue=NULL;
	gint queuerow = -1;
	gchar *fileold, *filenew, *directory, *foo;
	GdkColormap *colormap=NULL;
	GdkPixmap *gdkpixmap=NULL;
	GdkBitmap *mask=NULL;
	gchar *rowdata=NULL;
	GList *selection=NULL;
	gchar *temp=NULL;

	colormap = gtk_widget_get_colormap (gmp3);
	clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));
	
	directory = calloc(2000,sizeof(gchar));
	fileold = calloc(2000,sizeof(gchar));
	filenew = calloc(2000,sizeof(gchar));
	
	selection = clist_queue->selection;

	while (selection)
	{
		queuerow = (int)selection->data;
		if(strcmp(foo, "done")!=0)
		{
			gtk_clist_get_text (clist_queue, queuerow, 0, &foo);
			sprintf(directory, "%s", foo);
			*strrchr(directory,'/')='\0';
			sprintf(fileold, "%s", foo);
			
			gtk_clist_get_text (clist_queue, queuerow, 2, &foo);
			sprintf(filenew, "%s/%s", directory, foo);

			rowdata = gtk_clist_get_row_data(clist_queue, queuerow);

			if(rename(fileold, filenew)==0)
			{
				temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/done.xpm", NULL);
				gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
				free(temp);
				gtk_clist_set_pixmap(clist_queue, queuerow, 1, gdkpixmap, mask);
				sprintf(rowdata, "done");
			}
			else
			{
				temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/error.xpm", NULL);
				gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
				free(temp);
				gtk_clist_set_pixmap(GTK_CLIST (clist_queue), queuerow, 1, gdkpixmap, mask);
				sprintf(rowdata, "error");
			}
		}
		selection = selection->next;
	}	
	free(fileold);
	free(filenew);
	free(directory);
	gtk_clist_columns_autosize(clist_queue);
}












/*
 ***************************************************************************
 * will apply all rules specified by the user to selected files in queue.
 ***************************************************************************
 */
void queue_rules_apply_selected(gboolean preview)
{
	extern GtkWidget *gmp3;
	GtkCList *clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));
	GtkCList *clist_rules = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_rules")));
	gchar *filename, *extension, *output;
	gchar *rulestring;
	gchar *option[3];
	gint queuerow = -1;
	gint rulerow;
	gchar *fullfilename;
	GList *selection;

	gtk_clist_freeze(clist_queue);
	
	extension = calloc(2000,sizeof(gchar));
	filename = calloc(2000,sizeof(gchar));
	output = calloc(2000,sizeof(gchar));
	option[0] = calloc(2000,sizeof(gchar));
	option[1] = calloc(2000,sizeof(gchar));
	option[2] = calloc(2000,sizeof(gchar));
	
	selection = clist_queue->selection;

	while (selection)
	{
		queuerow = (int)selection->data;
		gtk_clist_get_text (clist_queue, queuerow, 0, &fullfilename);
		sprintf(extension, "%s", strrchr(fullfilename,'.'));
		sprintf(filename, "%s", strrchr(fullfilename,'/') + 1);
		*strrchr(filename,'.')='\0';
		
		rulerow=0;
		rulestring = gtk_clist_get_row_data (clist_rules, rulerow);
		while (rulestring)
		{
			sprintf(option[0], "%s", rulestring);
			rulestring=rulestring+strlen(rulestring)+1;
			sprintf(option[1], "%s", rulestring);
			rulestring=rulestring+strlen(rulestring)+1;
			sprintf(option[2], "%s", rulestring);

			if(!preview)
			{
// This rules can't be previewed (tag changers!)
				if(strcmp(option[0],(char *)"v1tov2")==0)
					apply_rule_v1tov2(fullfilename);
				if(strcmp(option[0],(char *)"v2tov1")==0)
					apply_rule_v2tov1(fullfilename);
				if(strcmp(option[0],(char *)"v1delete")==0)
				{
					if(access(fullfilename, W_OK)==0)
						del_id3v1_tag(fullfilename);
				}
				if(strcmp(option[0],(char *)"v2delete")==0)
				{
					if(access(fullfilename, W_OK)==0)
						del_id3v2_tag(fullfilename);
				}
				if(strcmp(option[0],(char *)"tagcleanup")==0)
					apply_rule_tagcleanup (fullfilename);
			}
			else {
// apply this rule to all filenames in queue.
				if(strcmp(option[0],(char *)"gettag")==0)
					apply_rule_gettag (filename, fullfilename, option[1]);
				if(strcmp(option[0],(char *)"replace")==0)
					apply_rule_replace (filename, option[1], option[2]);
				if(strcmp(option[0],(char *)"format")==0)
					apply_rule_format_to_format (filename, option[1], option[2]);
				if(strcmp(option[0],(char *)"parentheses")==0)
				{
					if(strcmp(option[1],(gchar *)"all")==0)
						apply_rule_remove_parentheses (filename, TRUE);
					else
						apply_rule_remove_parentheses (filename, FALSE);
				}
				if(strcmp(option[0],(char *)"forcecc")==0)
				{
					if(strcmp(option[1],(gchar *)"only first letter to upper")==0)
						apply_rule_forcecc (filename, 1);
					else if(strcmp(option[1],(gchar *)"all words begin uppercase")==0)
						apply_rule_forcecc (filename, 2);
					else if(strcmp(option[1],(gchar *)"every character lowercase")==0)
						apply_rule_forcecc (filename, 3);
					else
						apply_rule_forcecc (filename, 4);
				}
				if(strcmp(option[0],(char *)"deletechar")==0)
					apply_rule_delete_char (filename, atoi(option[1]), atoi(option[2]));
				if(strcmp(option[0],(char *)"increase")==0)
					apply_rule_number_increase (filename, atoi(option[1]));
				if(strcmp(option[0],(char *)"stripchar")==0)
					apply_rule_strip_to_digit (filename);
				if(strcmp(option[0],(char *)"twodigit")==0)
					apply_rule_digit_to_twodigit (filename);
				if(strcmp(option[0],(char *)"insertspace")==0)
					apply_rule_insert_char_before_uppercase (filename, (char *)" ");
				if(strcmp(option[0],(char *)"stripspace")==0)
					apply_rule_strip_multiple_spaces(filename);
				if(strcmp(option[0],(char *)"killspace")==0)
				{
					if(strcmp(option[1],(gchar *)"start and end")==0)
						apply_rule_wipe_spaces (filename, 2);
					else if(strcmp(option[1],(gchar *)"start")==0)
						apply_rule_wipe_spaces (filename, 0);
					else
						apply_rule_wipe_spaces (filename, 1);
				}
			}
			
			rulestring = (char *)gtk_clist_get_row_data (clist_rules, ++rulerow);
		}
		sprintf (output, "%s%s", filename, extension);
		gtk_clist_set_text (GTK_CLIST (clist_queue), queuerow, 2, (char *)output);
		selection = selection->next;
	}
	free(option[2]);
	free(option[1]);
	free(option[0]);
	free(output);
	free(filename);
	free(extension);

	gtk_clist_thaw(clist_queue);
	gtk_clist_columns_autosize(clist_queue);
}







void queue_files_add_recursive(gchar dir[4096])
{
	extern GtkWidget *gmp3;
	extern Stack clist_queue_data;
	GtkCList *clist_queue=NULL;
	
	struct dirent *pdirent=NULL;
	DIR *pdir=NULL;
	gchar *output[3];
	gchar *rowdata=NULL;
	gchar *foo=NULL, *temp=NULL;
	gchar bar[4096];
	gint row=0, i=-1;
	gboolean found;

	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	colormap = gtk_widget_get_colormap (gmp3);

	clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));

	gtk_clist_freeze(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));

	sprintf(bar, "Currently adding: %s", dir);
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, _("appbar"))), bar);
	while(g_main_iteration(FALSE));

// Open directory
	if((pdir = opendir (dir)) == NULL) 
	{
		printf("200 - Error opening requested path ('%s')!\n", dir);
		free(pdir);
		exit(1);
	}
	
	while((pdirent = readdir (pdir)) != NULL)
	{
		sprintf (bar, "%s%s", dir, pdirent->d_name);
		if ( is_a_dir (bar)
		    && strcmp (pdirent->d_name, "..")
		    && strcmp (pdirent->d_name, ".") )
		{
			strncat(bar, "/\0", 2);
			if(access(bar, R_OK)==0)
				queue_files_add_recursive(bar);
		}
		else {
			if( strrchr(bar, '.')
				&& !strncmp(strrchr(bar, '.'), ".mp3", 4) )
			{
// Make sure the entry is not in list, yet
				i=-1;
				found=FALSE;
				while ((gtk_clist_get_text (clist_queue, ++i, 0, &foo)))
				{
					if(strncmp(foo, bar, strlen(foo))==0)
					{
						found = TRUE;
						break;
					}
				}
				
				if(!found) {
					output[0] = mem_allocate(&clist_queue_data);
					output[2] = mem_allocate(&clist_queue_data);
					rowdata = mem_allocate(&clist_queue_data);
					sprintf (output[0], "%s%s", dir, pdirent->d_name);
					sprintf (output[2], "Click \"Preview\" to update.");
					sprintf (rowdata, "todo");
					row = gtk_clist_append (GTK_CLIST (clist_queue), output);
					
					temp=g_strconcat(PACKAGE_PIXMAPS_DIR, "/", "gmp3/todo.xpm", NULL);
					gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, temp);
					free(temp);
					
					gtk_clist_set_pixmap(GTK_CLIST (clist_queue), row, 1, gdkpixmap, mask);
					gtk_clist_set_row_data(clist_queue, row, rowdata);
				}
			}
		}
	}
	closedir(pdir);
	gtk_clist_sort(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));
	gtk_clist_columns_autosize(clist_queue);
	gtk_clist_thaw(GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue"))));
}






/*
 ***************************************************************************
 * will apply all rules specified by the user to a file as preview.
 ***************************************************************************
 */
void queue_rules_minipreview(void)
{
	extern GtkWidget *gmp3;
	extern GtkWidget *rules;
	GtkCList *clist_queue = GTK_CLIST (lookup_widget (gmp3, _("clist_gmp3_queue")));
	GtkCList *clist_rules = GTK_CLIST (lookup_widget (rules, _("clist_rules_rules")));
	gchar *filename, *extension, *output;
	gchar *rulestring;
	gchar *option[3];
	gint queuerow = -1;
	gint rulerow;
	gchar *fullfilename;

	queuerow = 0;
	if(!gtk_clist_get_text (clist_queue, queuerow, 0, &fullfilename)) return;
	gtk_clist_freeze(clist_queue);
	
	extension = calloc(2000,sizeof(gchar));
	filename = calloc(2000,sizeof(gchar));
	output = calloc(2000,sizeof(gchar));
	option[0] = calloc(2000,sizeof(gchar));
	option[1] = calloc(2000,sizeof(gchar));
	option[2] = calloc(2000,sizeof(gchar));
	
	sprintf(extension, "%s", strrchr(fullfilename,'.'));
	sprintf(filename, "%s", strrchr(fullfilename,'/') + 1);
	*strrchr(filename,'.')='\0';
	
	rulerow=0;
	rulestring = gtk_clist_get_row_data (clist_rules, rulerow);
	while (rulestring)
	{
		sprintf(option[0], "%s", rulestring);
		rulestring=rulestring+strlen(rulestring)+1;
		sprintf(option[1], "%s", rulestring);
		rulestring=rulestring+strlen(rulestring)+1;
		sprintf(option[2], "%s", rulestring);

// Apply rule to all filenames in queue.
		if(strcmp(option[0],(char *)"gettag")==0)
			apply_rule_gettag (filename, fullfilename, option[1]);
		if(strcmp(option[0],(char *)"replace")==0)
			apply_rule_replace (filename, option[1], option[2]);
		if(strcmp(option[0],(char *)"format")==0)
			apply_rule_format_to_format (filename, option[1], option[2]);
		if(strcmp(option[0],(char *)"parentheses")==0)
		{
			if(strcmp(option[1],(gchar *)"all")==0)
				apply_rule_remove_parentheses (filename, TRUE);
			else
				apply_rule_remove_parentheses (filename, FALSE);
		}
		if(strcmp(option[0],(char *)"forcecc")==0)
		{
			if(strcmp(option[1],(gchar *)"only first letter to upper")==0)
				apply_rule_forcecc (filename, 1);
			else if(strcmp(option[1],(gchar *)"all words begin uppercase")==0)
				apply_rule_forcecc (filename, 2);
			else if(strcmp(option[1],(gchar *)"every character lowercase")==0)
				apply_rule_forcecc (filename, 3);
			else
				apply_rule_forcecc (filename, 4);
		}
		if(strcmp(option[0],(char *)"deletechar")==0)
			apply_rule_delete_char (filename, atoi(option[1]), atoi(option[2]));
		if(strcmp(option[0],(char *)"increase")==0)
			apply_rule_number_increase (filename, atoi(option[1]));
		if(strcmp(option[0],(char *)"stripchar")==0)
			apply_rule_strip_to_digit (filename);
		if(strcmp(option[0],(char *)"twodigit")==0)
			apply_rule_digit_to_twodigit (filename);
		if(strcmp(option[0],(char *)"insertspace")==0)
			apply_rule_insert_char_before_uppercase (filename, (char *)" ");
		if(strcmp(option[0],(char *)"stripspace")==0)
			apply_rule_strip_multiple_spaces(filename);
		if(strcmp(option[0],(char *)"killspace")==0)
		{
			if(strcmp(option[1],(gchar *)"start and end")==0)
				apply_rule_wipe_spaces (filename, 2);
			else if(strcmp(option[1],(gchar *)"start")==0)
				apply_rule_wipe_spaces (filename, 0);
			else
				apply_rule_wipe_spaces (filename, 1);
		}
		
		rulestring = (char *)gtk_clist_get_row_data (clist_rules, ++rulerow);
	}
	gtk_clist_get_text (clist_queue, queuerow, 0, &fullfilename);
	sprintf (output, "Example. %s\n-> %s%s", strrchr(fullfilename, '/')+1, filename, extension);
	gtk_label_set_text (GTK_LABEL (lookup_widget(rules, "label_rules_example")), (char *)output);

	free(option[2]);
	free(option[1]);
	free(option[0]);
	free(output);
	free(filename);
	free(extension);

	gtk_clist_thaw(clist_queue);
	gtk_clist_columns_autosize(clist_queue);
}
