/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <ctype.h>
#include "rules.h"
#include "support.h"
#include "id3v1.h"
#include "id3v2.3.h"

/*
 ***************************************************************************
 * will strip all characters from the beginning of the string to the first occurence of a digit
 ***************************************************************************
 */
void apply_rule_strip_to_digit(gchar *source)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;

	if(source != NULL)
	{
		for(psource=source;(!isdigit(*psource) && *psource!='\0'); psource++);
		while(*psource!='\0')
			*destination++=*psource++;
	}
	if(strlen(temp)>0) sprintf(source, "%s", temp);
	free(temp);
	return;
}












/*
***************************************************************************
* this will remove parentheses. either all (all==TRUE) or only the first two.
***************************************************************************
*/
void apply_rule_remove_parentheses(gchar *source, gboolean all)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;

	gboolean firstopendone=FALSE;
	gboolean firstclosedone=FALSE;

	if(source != NULL)
	{
		for(psource = source; *psource != '\0'; psource++)
		{
			if( (*psource!='(')
			&& (*psource!=')') )
			{
				*destination++ = *psource;
			}
			else {
				if(all==FALSE)
				{
					if( (*psource=='(') && firstopendone )
						*destination++ = *psource;
					if( (*psource==')') && firstclosedone )
						*destination++ = *psource;

					if( (*psource=='(') && !firstopendone )
						firstopendone=TRUE;
					if( (*psource==')') && !firstclosedone )
						firstclosedone=TRUE;
				}
			}
		}
	}
	sprintf(source, "%s", temp);
	free(temp);
	return;
}











/*
***************************************************************************
* this will replace all multiple spaces by one space.
* spaces are:
* single space, tab, vertical tab, form feed, carriage return, or newline
***************************************************************************
*/
void apply_rule_strip_multiple_spaces(gchar *source)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;

	if(source != NULL)
	{
		for(psource = source; *psource != '\0'; psource++)
		{
			if( !isspace(*psource)
			|| (isspace(*psource) && !isspace(*(psource+1))) )
			{
				*destination++ = *psource;
			}
		}
	}
	sprintf(source, "%s", temp);
	free(temp);
	return;
}











/*
***************************************************************************
* this will set the first single(!!!) digit to twodigit.
***************************************************************************
*/
void apply_rule_digit_to_twodigit(gchar *source)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;

	if(source != NULL)
	{
		for(psource = source; *psource != '\0'; psource++)
		{
// found twodigit? then return without a change.
			if(isdigit(*psource) && isdigit(*(psource+1)))
			{
				free(temp);
				return;
			}

			if(isdigit(*psource))
			{
				*destination++ = '0';
				while(*psource != '\0')
					*destination++ = *psource++;
				break;
			}
			else {
				*destination++ = *psource;
			}
		}
	}
	sprintf(source, "%s", temp);
	free(temp);
	return;
}












/*
***************************************************************************
* this will insert a char before each uppercase character exept the first.
***************************************************************************
*/
void apply_rule_insert_char_before_uppercase(gchar *source, gchar *insert)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;

	if(source != NULL)
	{
		for(psource = source; *psource != '\0'; psource++)
		{
			if( isupper(*psource)
			&& (psource!=source))
			{
				*destination++ = *insert;
				while(*psource != '\0')
					*destination++ = *psource++;
				break;
			}
			else {
				*destination++ = *psource;
			}
		}
	}
	sprintf(source, "%s", temp);
	free(temp);
	return;
}











/*
 ***************************************************************************
 * should be self-explanatory
 ***************************************************************************
 */
void apply_rule_replace(gchar *source, gchar *from, gchar *to)
{
	gchar *psource;
	gchar *pto;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;
	
	if(source != NULL)
	{
		for(psource = source; *psource != '\0'; psource++)
		{
			if(strncmp(psource,from,strlen(from))==0) {
				pto=to;
				while(*pto!='\0')
					*destination++ = *pto++;
				psource+=strlen(from)-1;
			}
			else {
				*destination++ = *psource;
			}
		}
	}
	sprintf(source, "%s", temp);
	free(temp);
	return;
}












/*
***************************************************************************
* this will delete all chars in "source" from position "from" to "to".
***************************************************************************
*/
void apply_rule_delete_char(gchar *source, gint from, gint to)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;
	gint i=0;

	from--;
	to--;
	
	if(source != NULL)
	{
		for(psource = source; *(psource+i) != '\0'; i++)
		{
			if((i<from) || (i>to))
			{
				*destination++ = *(psource+i);
			}
		}
	}

	sprintf(source, "%s", temp);
	free(temp);
	return;
}














/*
***************************************************************************
* this will increase the first digit in source by number.
***************************************************************************
*/
void apply_rule_number_increase(gchar *source, gint number)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;
	gchar *digits = calloc(20,sizeof(gchar));
	gchar *digitspsource=digits;
	gchar *foo = calloc(20,sizeof(gchar));
	gchar *footemp=foo;
	gint i;

	if(source != NULL)
	{
		psource=source;
		while(!isdigit(*psource) && *psource!='\0')
		{
			*destination++ = *psource++;
		}
		if(*psource=='\0')
		{
			free(footemp);
			free(temp);
			free(digitspsource);
			return;
		}
		while(isdigit(*psource))
		{
			*digits++ = *psource++;
		}
		digits=digitspsource;
		i=(gint)atoi(digits);
		sprintf(foo, "%i", i+number);
		if(strlen(foo) < strlen(digits))
		{
			while(*foo!='\0') {
				digits++;
				*digits = *foo;
				foo++;
			}
		}
		else sprintf(digits, "%s", foo);
		digits=digitspsource;
		while(*digits!='\0')
		{
			*destination++=*digits++;
		}
		while(*psource!='\0')
		{
			*destination++=*psource++;
		}
	}

	sprintf(source, "%s", temp);
	free(footemp);
	free(temp);
	free(digitspsource);
	return;
}














/*
 ***************************************************************************
 * this will set the characters case:
 * mode=1:first word uppercase;
 * mode=2:every word uppercase;
 * mode=3:anything to lower;
 * mode=4:every character to upper.
 ***************************************************************************
 */
void apply_rule_forcecc(gchar *source, gint mode)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;
	gboolean done=FALSE;

	if(source != NULL)
	{
		for(psource = source; *psource != '\0'; psource++)
		{
			if(mode==1)
			{
				if( isalpha(*psource) && (done==FALSE) )
				{
					*destination++=toupper(*psource);
					done=TRUE;
				}
				else
					*destination++=tolower(*psource);
			}

			if(mode==2)
			{
				if( ( (!isalpha(*(psource-1)) && (*(psource-1)!='\'') && (*(psource-1)!=''))
						&& (isalpha(*psource)))
					|| (psource==source) )
					*destination++ = toupper(*psource);
				else
					*destination++=*psource;
			}

			if(mode==3)
			{
				if(isalpha(*psource))
					*destination++=tolower(*psource);
				else
					*destination++=*psource;
			}

			if(mode==4)
			{
				if(isalpha(*psource))
					*destination++=toupper(*psource);
				else
					*destination++=*psource;
			}
		}
	}

	sprintf(source, "%s", temp);
	free(temp);
	return;
}










/*
 ***************************************************************************
 * this will get data from an id3tag and store formatted in filename.
 ***************************************************************************
 */
void apply_rule_gettag(gchar *source, gchar *fullfilename, gchar *format)
{
	id3Tag *tag = calloc(2000,sizeof(id3Tag));
	id3Tag *v2 = calloc(2000,sizeof(id3Tag));
	gchar track[4];
	gboolean error=TRUE;

// get the tag to a struct.
	if(get_id3v1_tag(tag, fullfilename)==0)
		error = FALSE;
// v2 tag has higher prio, so it will overwrite the v1	
	if(get_id3v2_tag(v2, fullfilename)==0)
	{
		error = FALSE;
		v2->genre = tag->genre;
		free(tag);
		tag = v2;
	}
	else
		free(v2);
	
// If one of the requested options is not present, go back without a change.
	if((strstr(format, "%a")!=NULL) && (*tag->artist=='\0'))
		error=TRUE;
	if((strstr(format, "%s")!=NULL) && (*tag->title=='\0'))
		error=TRUE;
	if((strstr(format, "%b")!=NULL) && (*tag->album=='\0'))
		error=TRUE;
	if((strstr(format, "%y")!=NULL) && (*tag->year=='\0'))
		error=TRUE;
	if((strstr(format, "%t")!=NULL) && (tag->track==0))
		error=TRUE;

// set all data.
	if(!error)
	{
		apply_rule_replace(format, "%a", tag->artist);
		apply_rule_replace(format, "%s", tag->title);
		apply_rule_replace(format, "%b", tag->album);
		apply_rule_replace(format, "%y", tag->year);
		if(tag->track!=0) sprintf(track, "%u", (guint)tag->track);
		apply_rule_replace(format, "%t", track);
		apply_rule_replace(format, "%f", source);
	
		sprintf(source, "%s", format);
	}
	free(tag);
}







/*
 ***************************************************************************
 * this will kill spaces at the start/end/both of source.
 * spaces are:
 * single space, tab, vertical tab, form feed, carriage return, or newline
 * loc: 0-start 1-end 2-both
 ***************************************************************************
 */
void apply_rule_wipe_spaces(gchar *source, gint loc)
{
	gchar *psource;
	gchar *destination = calloc(2000,sizeof(gchar));
	gchar *temp=destination;

	if(source != NULL)
	{
		if((loc==0) || (loc==2))
		{
			strcpy(destination, source);
			while(*destination == ' ')
				destination++;
			sprintf(source, "%s", destination);
		}
		if((loc==1) || (loc==2))
		{
			for(psource = source+strlen(source)-1; *psource != '\0'; psource--)
			{
				if(isspace(*psource)) *psource='\0';
				else break;
			}
		}
	}
	free(temp);
	return;
}






/*
 ***************************************************************************
 * this will extract infos from the current filename and put
 * and put it together again with a new format.
 ***************************************************************************
 */
void apply_rule_format_to_format(gchar *filename, gchar *sformat, gchar *dformat)
{
	gint i=0, u=-1, f=0;
	
	gchar *trenner[8];
	gchar *ptrenner[8];
	gchar *option[8][2];
	gchar *psformat = sformat;
	gchar *pdformat = dformat;
	gchar *pfilename = filename;
	gchar foo[1000];
	gchar *pfoo = foo;
	gchar *destination = calloc(3000,sizeof(gchar));
	gchar *pdestination=destination;
	
	
	trenner[0] = calloc(2000,sizeof(gchar));
	ptrenner[0] = trenner[0];
	option[0][0] = calloc(2000,sizeof(gchar));
	option[0][1] = calloc(2000,sizeof(gchar));
	
/* Save the parts between %-switches in "trenner" and the switches in "options" */
	if(sformat != NULL)
	{
		for(psformat=sformat; (*psformat!='\0'); psformat++)
		{
			if(*psformat!='%')
			{
				*ptrenner[i]++=*psformat;
			}
			else
			{
				if(*(++psformat)=='%')
				{
					*ptrenner[i]++=*psformat;
				}
				else
				{
					*option[i][0] = *psformat;
					if(i+1>=7) break;
					trenner[++i] = calloc(2000,sizeof(gchar));
					ptrenner[i] = trenner[i];
					option[i][0] = calloc(2000,sizeof(gchar));
					option[i][1] = calloc(2000,sizeof(gchar));
				}
			}
		}
	
		pfilename=filename;
		while(++u<=i)
		{
			if(*option[u][0]=='\0') break;
			ptrenner[u] = trenner[u];
			for(; (*ptrenner[u]!='\0'); pfilename++)
			{
				if(*pfilename!=*ptrenner[u]++)
				{
					for(f=0; f<=i; f++)
					{
						free(trenner[f]);
						free(option[f][0]);
						free(option[f][1]);
					}
					free(destination);
					return;
				}
			}
			
			memset(foo, 0, 1000);
 			pfoo=foo;
			for(; (*pfilename!='\0'); pfilename++)
			{
				if(*trenner[u+1]=='\0')
				{
					strcpy(foo, pfilename);
					break;
				}
				if(strncmp(pfilename, trenner[u+1], strlen(trenner[u+1]))==0)
					break;
				else
					*pfoo++=*pfilename;
			}
			strcpy(option[u][1], foo);
		}
	}

	for(f=0; f<=i; f++)
		free(trenner[f]);
		
// So we got the source-parts nice formatted in 'options', we now figure out, how to fit them together again.
	u=-1;
	pdformat=dformat;
	while(++u<=i)
	{
		if(*option[u][0]=='\0') break;
/* browse through the format given by the user as desired destination */
		for(; (*pdformat!='\0'); )
		{
			// no '%' means, copy the char to the destination string
			if(*pdformat!='%')
				*(pdestination++)=*pdformat++;

			// This signs a valid option. if found, quit this for-loop!
			if(*pdformat=='%'
				&& *(pdformat+1)!='%')
				break;

			// a double '%' means just print a single '%' out to the destination string.
			if(*pdformat=='%'
				&& *(pdformat+1)=='%')
			{
				*(pdestination++)=*pdformat;
				pdformat+=2;
			}
		}

/* we arrive here if either a valid option marker (e.g. '%a') was found, or if the format-string ended
 * so we will look for which option was requested and append this part to the destination string */
		if(*(++pdformat)!='\0')
		{
			for(f=0;f<=i;f++)
			{
				if(*option[f][0]==*pdformat)
				{
					strncpy(pdestination, option[f][1], strlen(option[f][1]));
					pdestination+=strlen(option[f][1]);
					break;
				}
			}
			++pdformat;
		}
	}
	memset(filename, 0, strlen(destination)+1);
	strncpy(filename, destination, strlen(destination));
	for(u=0; u<=i; u++)
	{
		free(option[u][0]);
		free(option[u][1]);
	}
	free(destination);
}





/*
 * this will change padding spaces in an ID3V1tag to NULL
 */
void
apply_rule_tagcleanup(gchar *fullfilename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	
// Get old Tag
	memset(tag->artist, '\0', 1024);
	memset(tag->title, '\0', 1024);
	memset(tag->album, '\0', 1024);
	memset(tag->year, '\0', 1024);
	memset(tag->comment, '\0', 1024);
	get_id3v1_tag(tag, fullfilename);

	apply_rule_wipe_spaces(tag->artist, 1);
	apply_rule_wipe_spaces(tag->title, 1);
	apply_rule_wipe_spaces(tag->album, 1);
	apply_rule_wipe_spaces(tag->year, 1);
	apply_rule_wipe_spaces(tag->comment, 1);

// And put the changed tag to the file (if allowed)
	if(access(fullfilename, W_OK)==0)
		set_id3v1_tag(tag, fullfilename);
	
	free(tag);
}




/*
 * this will copy the contents of an ID3V1tag to the V2 tag.
 */
void
apply_rule_v1tov2(gchar *fullfilename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	
// Get old Tag
	memset(tag->artist, '\0', 1024);
	memset(tag->title, '\0', 1024);
	memset(tag->album, '\0', 1024);
	memset(tag->year, '\0', 1024);
	memset(tag->comment, '\0', 1024);
	if(get_id3v1_tag(tag, fullfilename)==0)
	{
		apply_rule_wipe_spaces(tag->artist, 1);
		apply_rule_wipe_spaces(tag->title, 1);
		apply_rule_wipe_spaces(tag->album, 1);
		apply_rule_wipe_spaces(tag->year, 1);
		apply_rule_wipe_spaces(tag->comment, 1);

// And put the changed tag to the file (if allowed)
		if(access(fullfilename, W_OK)==0)
			set_id3v2_tag(tag, fullfilename);
	}
	
	free(tag);
}





/*
 * this will copy the contents of an ID3V2tag to an V1 tag.
 */
void
apply_rule_v2tov1(gchar *fullfilename)
{
	id3Tag *tag = calloc(1,sizeof(id3Tag));
	
// And put the changed tag to the file (if allowed)
	if( (get_id3v2_tag(tag, fullfilename)==0)
		&& (access(fullfilename, W_OK)==0) )
		set_id3v1_tag(tag, fullfilename);
	
	free(tag);
}